/*
 * Copyright (c) 2011-2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __TIZEN_NETWORK_CONNECTION_EXTENSION_H__
#define __TIZEN_NETWORK_CONNECTION_EXTENSION_H__

#include "net_connection.h"
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file connection_extension.h
 */

/**
 * @brief Enumeration for extended network connection type.
 * @since_tizen 5.0
 */
typedef enum {
	CONNECTION_PROFILE_TYPE_MESH = CONNECTION_PROFILE_TYPE_BT + 1,  /**< Wi-Fi Mesh type */
} connection_profile_type_extended_e;

/**
 * @brief Start TCP Dump.
 *
 * @param[in] connection        The connection handle
 *
 * @return @c 0 on success, otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE                    Successful
 * @retval #CONNECTION_ERROR_OPERATION_FAILED        Operation failed
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER       Invalid parameter
*/
int connection_profile_start_tcpdump(connection_h connection);

/**
 * @brief Stop TCP Dump.
 *
 * @param[in] connection        The connection handle
 *
 * @return @c 0 on success, otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE                    Successful
 * @retval #CONNECTION_ERROR_OPERATION_FAILED        Operation failed
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER       Invalid parameter
*/
int connection_profile_stop_tcpdump(connection_h connection);

/**
 * @brief Get TCP Dump Running Status.
 *
 * @param[in] connection        The connection handle
 * @param[out] tcpdump_state    The state of tcpdump, tcpdump is in progress or not
 *
 * @return @c 0 on success, otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE                    Successful
 * @retval #CONNECTION_ERROR_OPERATION_FAILED        Operation failed
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER       Invalid parameter
*/
int connection_profile_get_tcpdump_state(connection_h connection, gboolean *tcpdump_state);

/**
 * @brief Creates a handle for managing data connections in C# API.
 * @since_tizen 5.0
 * @privlevel public
 * @privilege %http://tizen.org/privilege/network.get
 * @remarks You must release @a handle using connection_destroy_cs().
 * @param[in]  tid              TID in C#
 * @param[out] connection       The connection handle
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE                  Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_OUT_OF_MEMORY         Out of memory
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED     Permission denied
 * @see connection_destroy_cs()
 */
int connection_create_cs(int tid, connection_h *connection);

/**
 * @brief Destroys the connection handle in C# API.
 * @since_tizen 5.0
 * @param[in]  tid              TID in C#
 * @param[in] connection        The connection handle
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE                  Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @see connection_create_cs()
 */
int connection_destroy_cs(int tid, connection_h connection);

typedef enum {
	CONNECTION_MPTCP_DISABLE = 0,			// disable
	CONNECTION_MPTCP_ENABLE_ALL = 1,		// enable
	CONNECTION_MPTCP_ENABLE_SOCKOPT = 2,	// enable if the MPTCP_ENABLED socket option is set
} connection_mptcp_enable_e;

typedef enum {
	CONNECTION_MPTCP_PM_UNKNOWN,			// Unknown path manager
	CONNECTION_MPTCP_PM_DEFAULT,			// Default path manager - Do nothing
	CONNECTION_MPTCP_PM_FULLMESH,			// Full mesh of subflows
} connection_mptcp_path_manager_e;

typedef enum {
	CONNECTION_MPTCP_SCHEDULER_UNKNOWN,		// Unknown scheduler
	CONNECTION_MPTCP_SCHEDULER_DEFAULT,		// Default scheduler
	CONNECTION_MPTCP_SCHEDULER_ROUNDROBIN,	// Round robin scheduler
} connection_mptcp_scheduler_e;

/**
 * @brief Check if Multipath TCP is supported
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[out] supported	true if Multipath TCP is supported, false otherwise
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 */
int connection_mptcp_is_supported(connection_h connection, bool* supported);

/**
 * @brief Enable Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[in] enable	The enabled value
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER	Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_enable(connection_h connection, connection_mptcp_enable_e enable);

/**
 * @brief Disable Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_disable(connection_h connection);

/**
 * @brief Get the enabled value for Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[out] enable	The enabled value
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER	Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_get_enabled(connection_h connection, connection_mptcp_enable_e* enable);

/**
 * @brief Set the path manager of Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[in] pm	The path manager of Multipath TCP
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_set_path_manager(connection_h connection, connection_mptcp_path_manager_e pm);

/**
 * @brief Get the path manager of Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[out] pm	The path manager of Multipath TCP
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_get_path_manager(connection_h connection, connection_mptcp_path_manager_e* pm);

/**
 * @brief Set the path manager of Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[in] scheduler		The schedduler of Multipath TCP
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_set_scheduler(connection_h connection, connection_mptcp_scheduler_e scheduler);

/**
 * @brief Set the path manager of Multipath TCP
 * @since_tizen 5.0
 * @param[in] connection	The connection handle
 * @param[out] scheduler		The schedduler of Multipath TCP
 * @return @c 0 on success,
 *         otherwise a negative error value
 * @retval #CONNECTION_ERROR_NONE				Successful
 * @retval #CONNECTION_ERROR_INVALID_PARAMETER     Invalid parameter
 * @retval #CONNECTION_ERROR_NOT_SUPPORTED		Not supported
 * @retval #CONNECTION_ERROR_OPERATION_FAILED	Operation failed
 * @retval #CONNECTION_ERROR_PERMISSION_DENIED	Permission denied
 */
int connection_mptcp_get_scheduler(connection_h connection, connection_mptcp_scheduler_e* scheduler);

/**
* @}
*/


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __TIZEN_NETWORK_CONNECTION_EXTENSION_H__ */
