/*
 * Copyright (c) 2011-2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vconf/vconf.h>
#include <system_info.h>

#include "net_connection_private.h"

static GSList *conn_handle_list = NULL;
static int tv_profile = -1; // Unknown

//LCOV_EXCL_START
static int __connection_convert_net_state(int status)
{
	switch (status) {
	case VCONFKEY_NETWORK_CELLULAR:
		return CONNECTION_TYPE_CELLULAR;
	case VCONFKEY_NETWORK_WIFI:
		return CONNECTION_TYPE_WIFI;
	case VCONFKEY_NETWORK_ETHERNET:
		return CONNECTION_TYPE_ETHERNET;
	case VCONFKEY_NETWORK_BLUETOOTH:
		return CONNECTION_TYPE_BT;
	case VCONFKEY_NETWORK_DEFAULT_PROXY:
		return CONNECTION_TYPE_NET_PROXY;
	default:
		return CONNECTION_TYPE_DISCONNECTED;
	}
}

static int __connection_convert_cellular_state(int status)
{
	switch (status) {
	case VCONFKEY_NETWORK_CELLULAR_ON:
		return CONNECTION_CELLULAR_STATE_AVAILABLE;
	case VCONFKEY_NETWORK_CELLULAR_3G_OPTION_OFF:
		return CONNECTION_CELLULAR_STATE_CALL_ONLY_AVAILABLE;
	case VCONFKEY_NETWORK_CELLULAR_ROAMING_OFF:
		return CONNECTION_CELLULAR_STATE_ROAMING_OFF;
	case VCONFKEY_NETWORK_CELLULAR_FLIGHT_MODE:
		return CONNECTION_CELLULAR_STATE_FLIGHT_MODE;
	default:
		return CONNECTION_CELLULAR_STATE_OUT_OF_SERVICE;
	}
}

static bool __connection_check_handle_validity(connection_h connection)
{
	bool ret = false;

	if (connection == NULL)
		return false;

	if (g_slist_find(conn_handle_list, connection) != NULL)
		ret = true;

	return ret;
}

bool _connection_check_handle_validity(connection_h connection)
{
	return __connection_check_handle_validity(connection);
}

static void __connection_set_type_changed_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->type_changed_user_data = user_data;
	conn_handle->type_changed_callback = callback;
}

static void __connection_set_ip_changed_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->ip_changed_user_data = user_data;
	conn_handle->ip_changed_callback = callback;
}

static void __connection_set_proxy_changed_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->proxy_changed_user_data = user_data;
	conn_handle->proxy_changed_callback = callback;
}

static void __connection_set_internet_state_changed_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->internet_state_changed_user_data = user_data;
	conn_handle->internet_state_changed_callback = callback;
}

static void __connection_set_ethernet_cable_state_changed_cb(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->ethernet_cable_state_changed_callback = callback;
	conn_handle->ethernet_cable_state_changed_user_data = user_data;
}

static void __connection_set_default_cellular_service_profile_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->set_default_callback = callback;
	conn_handle->set_default_user_data = user_data;
}

static void __connection_open_profile_set_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->opened_callback = callback;
	conn_handle->opened_user_data = user_data;
}

static void __connection_close_profile_set_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->closed_callback = callback;
	conn_handle->closed_user_data = user_data;
}

static void __connection_reset_profile_set_callback(connection_handle_s *conn_handle,
			void *callback, void *user_data)
{
	conn_handle->reset_callback = callback;
	conn_handle->reset_user_data = user_data;
}
//LCOV_EXCL_STOP

/* Connection Manager ********************************************************/
EXPORT_API int connection_create(connection_h *connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	int rv;

	if (connection == NULL || __connection_check_handle_validity(*connection)) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	*connection = g_try_malloc0(sizeof(connection_handle_s));
	if (*connection != NULL) {
		CONNECTION_LOG(CONNECTION_INFO, "New handle created[%p]", *connection);
	} else {
		CONN_UNLOCK;
		return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
	}

	rv = _connection_libnet_init(*connection);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to create connection[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	conn_handle_list = g_slist_prepend(conn_handle_list, *connection);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_destroy(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	CONNECTION_LOG(CONNECTION_INFO, "Destroy handle: %p", connection);

	__connection_set_type_changed_callback(connection, NULL, NULL);
	__connection_set_ip_changed_callback(connection, NULL, NULL);
	__connection_set_proxy_changed_callback(connection, NULL, NULL);
	__connection_set_internet_state_changed_callback(connection, NULL, NULL);
	__connection_set_ethernet_cable_state_changed_cb(connection, NULL, NULL);

	conn_handle_list = g_slist_remove(conn_handle_list, connection);
	_connection_libnet_deinit(connection, (conn_handle_list == NULL));

	g_free(connection);
	connection = NULL;

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_create_cs(int tid, connection_h *connection)
{
	int rv;

	rv = connection_create(connection);
	if (rv == CONNECTION_ERROR_NONE) {
		CONN_LOCK;
		_connection_set_cs_tid(tid, *connection);
		CONN_UNLOCK;
	}

	return rv;
}

EXPORT_API int connection_destroy_cs(int tid, connection_h connection)
{
	int rv;

	CONN_LOCK;
	_connection_unset_cs_tid(tid, connection);
	CONN_UNLOCK;

	rv = connection_destroy(connection);

	return rv;
}

EXPORT_API int connection_get_type(connection_h connection, connection_type_e* type)
{
	int rv = 0;
	int status = 0;

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (type == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = vconf_get_int(VCONFKEY_NETWORK_STATUS, &status);
	if (rv != VCONF_OK) {
		CONNECTION_LOG(CONNECTION_ERROR, "vconf_get_int Failed = %d", status); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_INFO, "Connected Network = %d", status);

	*type = __connection_convert_net_state(status);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_ip_address(connection_h connection,
				connection_address_family_e address_family, char** ip_address)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (ip_address == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	switch (address_family) {
	case CONNECTION_ADDRESS_FAMILY_IPV4:
		*ip_address = vconf_get_str(VCONFKEY_NETWORK_IP);
		break;
	case CONNECTION_ADDRESS_FAMILY_IPV6:
		*ip_address = vconf_get_str(VCONFKEY_NETWORK_IP6);
		break;
	default:
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (*ip_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "vconf_get_str Failed"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED;//LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_proxy(connection_h connection,
				connection_address_family_e address_family, char** proxy)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (proxy == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	switch (address_family) {
	case CONNECTION_ADDRESS_FAMILY_IPV4:
	case CONNECTION_ADDRESS_FAMILY_IPV6:
		*proxy = vconf_get_str(VCONFKEY_NETWORK_PROXY);
		break;
	default:
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (*proxy == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "vconf_get_str Failed"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_mac_address(connection_h connection, connection_type_e type, char** mac_addr)
{
	FILE *fp;
	char buf[CONNECTION_MAC_INFO_LENGTH + 1];

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(WIFI_FEATURE, ETHERNET_FEATURE);

	if (type == CONNECTION_TYPE_WIFI)
		CHECK_FEATURE_SUPPORTED(WIFI_FEATURE);
	else if (type == CONNECTION_TYPE_ETHERNET) //LCOV_EXCL_LINE
		CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE); //LCOV_EXCL_LINE

	if (mac_addr == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	switch (type) {
	case CONNECTION_TYPE_WIFI:
		if (__builtin_expect(tv_profile == -1, 0)) {
			char *profileName;
			system_info_get_platform_string("http://tizen.org/feature/profile", &profileName);
			if (*profileName == 't' || *profileName == 'T')
				tv_profile = 1;
			else
				tv_profile = 0;
			free(profileName);
		}
		if (tv_profile == 1) {
			fp = fopen(WIFI_MAC_INFO_FILE, "r");
			if (fp == NULL) {
				CONNECTION_LOG(CONNECTION_ERROR, "Failed to open file %s", WIFI_MAC_INFO_FILE); //LCOV_EXCL_LINE
				CONN_UNLOCK;
				return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
			}

			if (fgets(buf, sizeof(buf), fp) == NULL) {
				CONNECTION_LOG(CONNECTION_ERROR, "Failed to get MAC info from %s", WIFI_MAC_INFO_FILE); //LCOV_EXCL_LINE
				fclose(fp); //LCOV_EXCL_LINE
				CONN_UNLOCK;
				return CONNECTION_ERROR_OPERATION_FAILED;
			}

			CONNECTION_LOG(CONNECTION_INFO, "%s : %s", WIFI_MAC_INFO_FILE, buf);

			*mac_addr = (char *)malloc(CONNECTION_MAC_INFO_LENGTH + 1);
			if (*mac_addr == NULL) {
				CONNECTION_LOG(CONNECTION_ERROR, "malloc() failed"); //LCOV_EXCL_LINE
				fclose(fp); //LCOV_EXCL_LINE
				CONN_UNLOCK;
				return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
			}
			g_strlcpy(*mac_addr, buf, CONNECTION_MAC_INFO_LENGTH + 1);
			fclose(fp);
		} else {
			*mac_addr = vconf_get_str(VCONFKEY_WIFI_BSSID_ADDRESS);

			if (*mac_addr == NULL) {
				CONNECTION_LOG(CONNECTION_ERROR, "Failed to get vconf from %s", VCONFKEY_WIFI_BSSID_ADDRESS); //LCOV_EXCL_LINE
				CONN_UNLOCK;
				return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
			}
		}
		break;
	//LCOV_EXCL_START
	case CONNECTION_TYPE_ETHERNET:
		fp = fopen(ETHERNET_MAC_INFO_FILE, "r");
		if (fp == NULL) {
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to open file %s", ETHERNET_MAC_INFO_FILE);
			CONN_UNLOCK;
			return CONNECTION_ERROR_OUT_OF_MEMORY;
		}

		if (fgets(buf, sizeof(buf), fp) == NULL) {
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to get MAC info from %s", ETHERNET_MAC_INFO_FILE);
			fclose(fp);
			CONN_UNLOCK;
			return CONNECTION_ERROR_OPERATION_FAILED;
		}

		CONNECTION_LOG(CONNECTION_INFO, "%s : %s", ETHERNET_MAC_INFO_FILE, buf);

		*mac_addr = (char *)malloc(CONNECTION_MAC_INFO_LENGTH + 1);
		if (*mac_addr == NULL) {
			CONNECTION_LOG(CONNECTION_ERROR, "malloc() failed");
			fclose(fp);
			CONN_UNLOCK;
			return CONNECTION_ERROR_OUT_OF_MEMORY;
		}

		g_strlcpy(*mac_addr, buf, CONNECTION_MAC_INFO_LENGTH + 1);
		fclose(fp);

		break;
	//LCOV_EXCL_STOP
	default:
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	/* Checking Invalid MAC Address */
	if ((strcmp(*mac_addr, "00:00:00:00:00:00") == 0) ||
			(strcmp(*mac_addr, "ff:ff:ff:ff:ff:ff") == 0)) {
		CONNECTION_LOG(CONNECTION_ERROR, "MAC Address(%s) is invalid", *mac_addr); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_OPERATION; //LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_INFO, "MAC Address %s", *mac_addr);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}


EXPORT_API int connection_is_metered_network(connection_h connection, bool* is_metered)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (is_metered == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_metered_state(connection, is_metered);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get metered state[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_INFO, "metered state: %s", is_metered ? "true" : "false");
	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}


EXPORT_API int connection_get_cellular_state(connection_h connection, connection_cellular_state_e* state)
{
	int rv = 0;
	int status = 0;
	int cellular_state = 0;
#if defined TIZEN_DUALSIM_ENABLE
	int sim_id = 0;
#endif

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	if (state == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_STATE, &status);
	if (rv != VCONF_OK) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get cellular state"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_INFO, "Cellular: %d", status);
	*state = __connection_convert_cellular_state(status);

	if (*state == CONNECTION_CELLULAR_STATE_AVAILABLE) {
#if defined TIZEN_DUALSIM_ENABLE
		rv = vconf_get_int(VCONF_TELEPHONY_DEFAULT_DATA_SERVICE, &sim_id);
		if (rv != VCONF_OK) {
			CONNECTION_LOG(CONNECTION_ERROR,
					"Failed to get default subscriber id", sim_id);
			CONN_UNLOCK;
			return CONNECTION_ERROR_OPERATION_FAILED;
		}

		switch (sim_id) {
		case CONNECTION_CELLULAR_SUBSCRIBER_1:
#endif
			rv = vconf_get_int(VCONFKEY_DNET_STATE, &cellular_state);
#if defined TIZEN_DUALSIM_ENABLE
			break;

		case CONNECTION_CELLULAR_SUBSCRIBER_2:
			rv = vconf_get_int(VCONFKEY_DNET_STATE2, &cellular_state);
			break;

		default:
			CONNECTION_LOG(CONNECTION_ERROR, "Invalid subscriber id:%d", sim_id);
			CONN_UNLOCK;
			return CONNECTION_ERROR_OPERATION_FAILED;
		}
#endif
		if (rv != VCONF_OK) {
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to get cellular state"); //LCOV_EXCL_LINE
			CONN_UNLOCK;
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
		}
	}

	CONNECTION_LOG(CONNECTION_INFO, "Cellular state: %d", cellular_state);

	if (cellular_state == VCONFKEY_DNET_NORMAL_CONNECTED ||
			cellular_state == VCONFKEY_DNET_SECURE_CONNECTED ||
			cellular_state == VCONFKEY_DNET_TRANSFER)
		*state = CONNECTION_CELLULAR_STATE_CONNECTED;

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_wifi_state(connection_h connection, connection_wifi_state_e* state)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(WIFI_FEATURE);

	if (state == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_wifi_state(connection, state);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get Wi-Fi state[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_INFO, "Wi-Fi state: %d", *state);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

//LCOV_EXCL_START
EXPORT_API int connection_get_ethernet_state(connection_h connection, connection_ethernet_state_e *state)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (state == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_ethernet_state(connection, state);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get ethernet state[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_ethernet_cable_state(connection_h connection, connection_ethernet_cable_state_e *state)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (state == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_ethernet_cable_state(connection, state);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get ethernet cable state[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_ethernet_cable_state_chaged_cb(connection_h connection,
			  connection_ethernet_cable_state_chaged_cb callback, void *user_data)
{
	DEPRECATED_LOG(__FUNCTION__, "connection_set_ethernet_cable_state_changed_cb");

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	DEPRECATED_LOG("connection_ethernet_cable_state_chaged_cb",
			"connection_ethernet_cable_state_changed_cb");

	__connection_set_ethernet_cable_state_changed_cb(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_ethernet_cable_state_chaged_cb(connection_h connection)
{
	DEPRECATED_LOG(__FUNCTION__, "connection_unset_ethernet_cable_state_changed_cb");

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_ethernet_cable_state_changed_cb(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_ethernet_cable_state_changed_cb(connection_h connection,
			  connection_ethernet_cable_state_changed_cb callback, void *user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_ethernet_cable_state_changed_cb(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_ethernet_cable_state_changed_cb(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_ethernet_cable_state_changed_cb(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}
//LCOV_EXCL_STOP

EXPORT_API int connection_get_bt_state(connection_h connection, connection_bt_state_e *state)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TETHERING_BLUETOOTH_FEATURE);

	if (state == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_bluetooth_state(connection, state);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get bluetooth state[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_type_changed_cb(connection_h connection,
					connection_type_changed_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_type_changed_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_type_changed_cb(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_type_changed_callback(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_ip_address_changed_cb(connection_h connection,
				connection_address_changed_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_ip_changed_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_ip_address_changed_cb(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_ip_changed_callback(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_proxy_address_changed_cb(connection_h connection,
				connection_address_changed_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_proxy_changed_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_proxy_address_changed_cb(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_proxy_changed_callback(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_internet_state_changed_cb(connection_h connection,
		connection_internet_state_changed_cb callback, void *user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (callback == NULL || !(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_internet_state_changed_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_unset_internet_state_changed_cb(connection_h connection)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	__connection_set_internet_state_changed_callback(connection, NULL, NULL);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_add_profile(connection_h connection, connection_profile_h profile)
{
	int rv = 0;
	connection_handle_s *conn_handle = (connection_handle_s *)connection;
	net_profile_info_t *profile_info = (net_profile_info_t *)profile;

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    !(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (profile_info->profile_type != NET_DEVICE_CELLULAR) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	if (profile_info->ProfileInfo.Pdp.PSModemPath[0] != '/' ||
			strlen(profile_info->ProfileInfo.Pdp.PSModemPath) < 2) {
		CONNECTION_LOG(CONNECTION_ERROR, "Modem object path is NULL"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_add_profile(conn_handle->network_info_handle,
				profile_info->ProfileInfo.Pdp.ServiceType, profile_info);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to add profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_remove_profile(connection_h connection, connection_profile_h profile)
{
	int rv = 0;
	connection_handle_s *conn_handle = (connection_handle_s *)connection;
	net_profile_info_t *profile_info = (net_profile_info_t *)profile;

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
		!(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (profile_info->profile_type != NET_DEVICE_CELLULAR &&
		profile_info->profile_type != NET_DEVICE_MESH &&
	    profile_info->profile_type != NET_DEVICE_WIFI) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_delete_profile(conn_handle->network_info_handle, profile_info->ProfileName);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to delete profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_update_profile(connection_h connection, connection_profile_h profile)
{
	int rv = 0;
	connection_handle_s *conn_handle = (connection_handle_s *)connection;
	net_profile_info_t *profile_info = (net_profile_info_t *)profile;

	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    !(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = net_modify_profile(conn_handle->network_info_handle,
			profile_info->ProfileName, (net_profile_info_t*)profile);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to modify profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_profile_iterator(connection_h connection,
		connection_iterator_type_e type, connection_profile_iterator_h* profile_iterator)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    (type != CONNECTION_ITERATOR_TYPE_REGISTERED &&
	     type != CONNECTION_ITERATOR_TYPE_CONNECTED &&
	     type != CONNECTION_ITERATOR_TYPE_DEFAULT)) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_profile_iterator(connection, type, profile_iterator);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get profile iterator [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_profile_iterator_next(connection_profile_iterator_h profile_iterator,
							connection_profile_h* profile)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	int rv = _connection_libnet_get_iterator_next(profile_iterator, profile);

	CONN_UNLOCK;
	return rv;
}

EXPORT_API bool connection_profile_iterator_has_next(connection_profile_iterator_h profile_iterator)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	int rv = _connection_libnet_iterator_has_next(profile_iterator);

	CONN_UNLOCK;
	return rv;
}

EXPORT_API int connection_destroy_profile_iterator(connection_profile_iterator_h profile_iterator)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	int rv = _connection_libnet_destroy_iterator(profile_iterator);

	CONN_UNLOCK;
	return rv;
}

EXPORT_API int connection_get_current_profile(connection_h connection, connection_profile_h* profile)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) || profile == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_current_profile(connection, profile);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get current profile [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_default_cellular_service_profile(
		connection_h connection, connection_cellular_service_type_e type,
		connection_profile_h *profile)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	if (!(__connection_check_handle_validity(connection)) || profile == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_get_cellular_service_profile(connection, type, profile);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get default cellular service profile [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_default_cellular_service_profile(connection_h connection,
		connection_cellular_service_type_e type, connection_profile_h profile)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	if (!(__connection_check_handle_validity(connection)) || profile == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_set_cellular_service_profile_sync(connection, type, profile);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to set default cellular service profile [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_set_default_cellular_service_profile_async(connection_h connection,
		connection_cellular_service_type_e type, connection_profile_h profile,
		connection_set_default_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	int rv;

	if (!(__connection_check_handle_validity(connection)) ||
	    profile == NULL || callback == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = _connection_libnet_set_cellular_service_profile_async(connection, type, profile);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to set default cellular service profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	__connection_set_default_cellular_service_profile_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_open_profile(connection_h connection, connection_profile_h profile,
					connection_opened_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE);

	int rv;

	if (!(__connection_check_handle_validity(connection)) ||
	    profile == NULL || callback == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = _connection_libnet_open_profile(connection, profile);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to open profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	__connection_open_profile_set_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_close_profile(connection_h connection, connection_profile_h profile,
					connection_closed_cb callback, void* user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE);

	int rv;

	if (!(__connection_check_handle_validity(connection)) ||
	    profile == NULL || callback == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = _connection_libnet_close_profile(connection, profile);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to close profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	__connection_close_profile_set_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_reset_profile(connection_h connection,
				connection_reset_option_e type, int id, connection_reset_cb callback, void *user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);

	int rv;

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Wrong Parameter Passed");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (id < 0 || id > 1) {
		CONNECTION_LOG(CONNECTION_ERROR, "Wrong Parameter Passed"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = _connection_libnet_reset_profile(connection, type, id);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to reset profile[%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	__connection_reset_profile_set_callback(connection, callback, user_data);

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_add_route(connection_h connection, const char* interface_name, const char* host_address)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_add_route(connection, interface_name, host_address);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to add route [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_remove_route(connection_h connection, const char* interface_name, const char* host_address)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_remove_route(connection, interface_name, host_address);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to remove route [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_add_route_ipv6(connection_h connection, const char *interface_name, const char *host_address, const char * gateway)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_add_route_ipv6(connection, interface_name, host_address, gateway);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to add route ipv6 [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_remove_route_ipv6(connection_h connection, const char *interface_name, const char *host_address, const char * gateway)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = _connection_libnet_remove_route_ipv6(connection, interface_name, host_address, gateway);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to remove route ipv6 [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_add_route_entry(connection_h connection,
		connection_address_family_e address_family,	const char *interface_name,
		const char *host_address, const char *gateway)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
		(address_family != CONNECTION_ADDRESS_FAMILY_IPV4 &&
	     address_family != CONNECTION_ADDRESS_FAMILY_IPV6) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4) {
		int rv = _connection_libnet_add_route_entry(connection, CONNECTION_ADDRESS_FAMILY_IPV4,
								interface_name, host_address, gateway);
		CONN_UNLOCK;
		return rv;
	} else {
		int rv = _connection_libnet_add_route_entry(connection, CONNECTION_ADDRESS_FAMILY_IPV6,
								interface_name, host_address, gateway);
		CONN_UNLOCK;
		return rv;
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_remove_route_entry(connection_h connection,
		connection_address_family_e address_family,	const char *interface_name,
		const char *host_address, const char *gateway)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE, ETHERNET_FEATURE);

	if (!(__connection_check_handle_validity(connection)) ||
		(address_family != CONNECTION_ADDRESS_FAMILY_IPV4 &&
	     address_family != CONNECTION_ADDRESS_FAMILY_IPV6) ||
	    interface_name == NULL || host_address == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4) {
		int rv = _connection_libnet_remove_route_entry(connection, CONNECTION_ADDRESS_FAMILY_IPV4,
								interface_name, host_address, gateway);
		CONN_UNLOCK;
		return rv;
	} else {
		int rv = _connection_libnet_remove_route_entry(connection, CONNECTION_ADDRESS_FAMILY_IPV6,
								interface_name, host_address, gateway);
		CONN_UNLOCK;
		return rv;
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

static int __get_cellular_statistic(connection_statistics_type_e statistics_type, long long *llsize)
{
	int rv = VCONF_OK, rv1 = VCONF_OK;
	int last_size = 0, size = 0;
#if defined TIZEN_DUALSIM_ENABLE
	int sim_id = 0;
#endif

	if (llsize == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	switch (statistics_type) {
	case CONNECTION_STATISTICS_TYPE_LAST_SENT_DATA:
	case CONNECTION_STATISTICS_TYPE_LAST_RECEIVED_DATA:
	case CONNECTION_STATISTICS_TYPE_TOTAL_SENT_DATA:
	case CONNECTION_STATISTICS_TYPE_TOTAL_RECEIVED_DATA:
		break;
	default:
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

#if defined TIZEN_DUALSIM_ENABLE
	rv = vconf_get_int(VCONF_TELEPHONY_DEFAULT_DATA_SERVICE, &sim_id);
	if (rv != VCONF_OK) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get default subscriber id");
		*llsize = 0;
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	switch (sim_id) {
	case 0:
#endif
		switch (statistics_type) {
		case CONNECTION_STATISTICS_TYPE_LAST_SENT_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_SNT, &last_size);
			break;
		case CONNECTION_STATISTICS_TYPE_LAST_RECEIVED_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_RCV, &last_size);
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_SENT_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_SNT, &last_size);
			rv1 = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_TOTAL_SNT, &size);
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_RECEIVED_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_RCV, &last_size);
			rv1 = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_TOTAL_RCV, &size);
			break;
		}
#if defined TIZEN_DUALSIM_ENABLE
		break;
	case 1:
		switch (statistics_type) {
		case CONNECTION_STATISTICS_TYPE_LAST_SENT_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_SNT2, &last_size);
			break;
		case CONNECTION_STATISTICS_TYPE_LAST_RECEIVED_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_RCV2, &last_size);
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_SENT_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_SNT2, &last_size);
			rv1 = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_TOTAL_SNT2, &size);
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_RECEIVED_DATA:
			rv = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_LAST_RCV2, &last_size);
			rv1 = vconf_get_int(VCONFKEY_NETWORK_CELLULAR_PKT_TOTAL_RCV2, &size);
			break;
		}
		break;
	default:
		*llsize = 0;
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid subscriber id:%d", sim_id);
		return CONNECTION_ERROR_OPERATION_FAILED;
	}
#endif

	if (rv != VCONF_OK || rv1 != VCONF_OK) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get cellular statistics"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	*llsize = (long long)(last_size * 1000) + (long long)(size * 1000);
	CONNECTION_LOG(CONNECTION_INFO, "%lld bytes", *llsize);

	return CONNECTION_ERROR_NONE;
}

static int __get_statistic(connection_handle_s *conn_handle, connection_type_e connection_type,
		connection_statistics_type_e statistics_type, long long *llsize)
{
	int rv, stat_type;
	unsigned long long ull_size;

	if (llsize == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv  = _connection_libnet_check_get_privilege();
	if (rv == CONNECTION_ERROR_PERMISSION_DENIED)
		return rv;
	else if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get statistics"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	if (connection_type == CONNECTION_TYPE_CELLULAR)
		return __get_cellular_statistic(statistics_type, llsize);
	else if (connection_type == CONNECTION_TYPE_WIFI) {
		switch (statistics_type) {
		case CONNECTION_STATISTICS_TYPE_LAST_SENT_DATA:
			stat_type = NET_STATISTICS_TYPE_LAST_SENT_DATA;
			break;
		case CONNECTION_STATISTICS_TYPE_LAST_RECEIVED_DATA:
			stat_type = NET_STATISTICS_TYPE_LAST_RECEIVED_DATA;
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_SENT_DATA:
			stat_type = NET_STATISTICS_TYPE_TOTAL_SENT_DATA;
			break;
		case CONNECTION_STATISTICS_TYPE_TOTAL_RECEIVED_DATA:
			stat_type = NET_STATISTICS_TYPE_TOTAL_RECEIVED_DATA;
			break;
		default:
			return CONNECTION_ERROR_INVALID_PARAMETER;
		}

		rv  = _connection_libnet_get_statistics(conn_handle, stat_type, &ull_size);
		if (rv == CONNECTION_ERROR_PERMISSION_DENIED)
			return rv;
		else if (rv != CONNECTION_ERROR_NONE) {
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to get Wi-Fi statistics"); //LCOV_EXCL_LINE
			*llsize = 0; //LCOV_EXCL_LINE
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
		}

		CONNECTION_LOG(CONNECTION_INFO, "%lld bytes", ull_size);
		*llsize = (long long)ull_size;
	} else
		return CONNECTION_ERROR_INVALID_PARAMETER;

	return CONNECTION_ERROR_NONE;
}

static int __reset_statistic(connection_handle_s *conn_handle,
			connection_type_e connection_type, connection_statistics_type_e statistics_type)
{
	int conn_type;
	int stat_type;
	int rv;

	if (connection_type == CONNECTION_TYPE_CELLULAR)
		conn_type = NET_DEVICE_CELLULAR;
	else if (connection_type == CONNECTION_TYPE_WIFI)
		conn_type = NET_DEVICE_WIFI;
	else
		return CONNECTION_ERROR_INVALID_PARAMETER;

	switch (statistics_type) {
	case CONNECTION_STATISTICS_TYPE_LAST_SENT_DATA:
		stat_type = NET_STATISTICS_TYPE_LAST_SENT_DATA;
		break;
	case CONNECTION_STATISTICS_TYPE_LAST_RECEIVED_DATA:
		stat_type = NET_STATISTICS_TYPE_LAST_RECEIVED_DATA;
		break;
	case CONNECTION_STATISTICS_TYPE_TOTAL_SENT_DATA:
		stat_type = NET_STATISTICS_TYPE_TOTAL_SENT_DATA;
		break;
	case CONNECTION_STATISTICS_TYPE_TOTAL_RECEIVED_DATA:
		stat_type = NET_STATISTICS_TYPE_TOTAL_RECEIVED_DATA;
		break;
	default:
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = _connection_libnet_set_statistics(conn_handle, conn_type, stat_type);
	if (rv != CONNECTION_ERROR_NONE)
		return rv;

	CONNECTION_LOG(CONNECTION_INFO, "connection_reset_statistics success");

	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_get_statistics(connection_h connection,
				connection_type_e connection_type,
				connection_statistics_type_e statistics_type, long long* size)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE);

	if (connection_type == CONNECTION_TYPE_CELLULAR)
		CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);
	else if (connection_type == CONNECTION_TYPE_WIFI)
		CHECK_FEATURE_SUPPORTED(WIFI_FEATURE);

	if (!(__connection_check_handle_validity(connection)) || size == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = __get_statistic(connection, connection_type, statistics_type, size);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get statistics [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_reset_statistics(connection_h connection,
				connection_type_e connection_type,
				connection_statistics_type_e statistics_type)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE);

	if (connection_type == CONNECTION_TYPE_CELLULAR)
		CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE);
	else if (connection_type == CONNECTION_TYPE_WIFI)
		CHECK_FEATURE_SUPPORTED(WIFI_FEATURE);

	if (!__connection_check_handle_validity(connection)) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = __reset_statistic(connection, connection_type, statistics_type);
	if (rv != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Fail to get statistics [%d]", rv); //LCOV_EXCL_LINE
		CONN_UNLOCK;
		return rv; //LCOV_EXCL_LINE
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_foreach_ipv6_address(connection_h connection,
		connection_type_e connection_type, connection_ipv6_address_cb callback,
		void *user_data)
{
	CONN_LOCK;

	CHECK_FEATURE_SUPPORTED(TELEPHONY_FEATURE, WIFI_FEATURE,
			TETHERING_BLUETOOTH_FEATURE, ETHERNET_FEATURE);

	GSList *ipv6_address_list = NULL;

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	int rv = CONNECTION_ERROR_NONE;

	switch (connection_type) {
	case CONNECTION_TYPE_WIFI:
		rv = net_foreach_ipv6_address(NET_DEVICE_WIFI,
				&ipv6_address_list);
		break;
	case CONNECTION_TYPE_CELLULAR:
		rv = net_foreach_ipv6_address(NET_DEVICE_CELLULAR,
				&ipv6_address_list);
		break;
	case CONNECTION_TYPE_ETHERNET:
		rv = net_foreach_ipv6_address(NET_DEVICE_ETHERNET,
				&ipv6_address_list);
		break;
	case CONNECTION_TYPE_BT:
		rv = net_foreach_ipv6_address(NET_DEVICE_BLUETOOTH,
				&ipv6_address_list);
		break;
	default:
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "net_get_multiple_id_address"
				" Failed = %d\n", rv);
		CONN_UNLOCK;
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	GSList *list;
	for (list = ipv6_address_list; list; list = list->next) {
		rv = callback((char *)list->data, user_data);
		if (rv == false)
			break;
	}

	g_slist_free_full(ipv6_address_list, g_free);
	ipv6_address_list = NULL;

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_profile_start_tcpdump(connection_h connection)
{
	int ret = 0;

	CONN_LOCK;

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	ret = _connection_libnet_start_tcpdump(connection);
	if (ret != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to start tcpdump (%d)", ret);
		CONN_UNLOCK;
		return ret;
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_profile_stop_tcpdump(connection_h connection)
{
	int ret = 0;

	CONN_LOCK;

	if (!(__connection_check_handle_validity(connection))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	ret = _connection_libnet_stop_tcpdump(connection);
	if (ret != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to stop tcpdump (%d)", ret);
		CONN_UNLOCK;
		return ret;
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}

EXPORT_API int connection_profile_get_tcpdump_state(connection_h connection, gboolean *tcpdump_state)
{
	int ret = 0;

	CONN_LOCK;

	if (!(__connection_check_handle_validity(connection)) || !tcpdump_state) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter");
		CONN_UNLOCK;
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	ret = _connection_libnet_get_tcpdump_state(connection, tcpdump_state);
	if (ret != CONNECTION_ERROR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get the tcpdump state (%d)", ret);
		CONN_UNLOCK;
		return ret;
	}

	CONN_UNLOCK;
	return CONNECTION_ERROR_NONE;
}
