/*
 * Copyright (c) 2011-2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <vconf/vconf.h>
#include <system_info.h>
#include <arpa/inet.h>

#include "net_connection_private.h"

static GSList *prof_handle_list = NULL;
static GHashTable *profile_cb_table = NULL;
static pthread_mutex_t g_conn_thread_mutex = PTHREAD_MUTEX_INITIALIZER;

struct _profile_cb_s {
	connection_profile_state_changed_cb callback;
	connection_profile_state_e state;
	void *user_data;
};

struct _profile_list_s {
	int next;
	int count;
	net_profile_info_t *profiles;
};

static struct _profile_list_s profile_iterator = {0, 0, NULL};
static bool connection_is_feature_checked[CONNECTION_SUPPORTED_FEATURE_MAX] = {0, };
static bool connection_feature_supported[CONNECTION_SUPPORTED_FEATURE_MAX] = {0, };

//LCOV_EXCL_START
static connection_error_e __libnet_convert_to_cp_error_type(net_err_t err_type)
{
	switch (err_type) {
	case NET_ERR_NONE:
		return CONNECTION_ERROR_NONE;
	case NET_ERR_APP_ALREADY_REGISTERED:
		return CONNECTION_ERROR_INVALID_OPERATION;
	case NET_ERR_APP_NOT_REGISTERED:
		return CONNECTION_ERROR_INVALID_OPERATION;
	case NET_ERR_NO_ACTIVE_CONNECTIONS:
		return CONNECTION_ERROR_NO_CONNECTION;
	case NET_ERR_ACTIVE_CONNECTION_EXISTS:
		return CONNECTION_ERROR_ALREADY_EXISTS;
	case NET_ERR_CONNECTION_DHCP_FAILED:
		return CONNECTION_ERROR_DHCP_FAILED;
	case NET_ERR_CONNECTION_INVALID_KEY:
		return CONNECTION_ERROR_INVALID_KEY;
	case NET_ERR_IN_PROGRESS:
		return CONNECTION_ERROR_NOW_IN_PROGRESS;
	case NET_ERR_OPERATION_ABORTED:
		return CONNECTION_ERROR_OPERATION_ABORTED;
	case NET_ERR_TIME_OUT:
		return CONNECTION_ERROR_NO_REPLY;
	case NET_ERR_ACCESS_DENIED:
		return CONNECTION_ERROR_PERMISSION_DENIED;
	default:
		return CONNECTION_ERROR_OPERATION_FAILED;
	}
}

static const char *__libnet_convert_cp_error_type_to_string(connection_error_e err_type)
{
	switch (err_type) {
	case CONNECTION_ERROR_NONE:
		return "NONE";
	case CONNECTION_ERROR_INVALID_PARAMETER:
		return "INVALID_PARAMETER";
	case CONNECTION_ERROR_OUT_OF_MEMORY:
		return "OUT_OF_MEMORY";
	case CONNECTION_ERROR_INVALID_OPERATION:
		return "INVALID_OPERATION";
	case CONNECTION_ERROR_ADDRESS_FAMILY_NOT_SUPPORTED:
		return "ADDRESS_FAMILY_NOT_SUPPORTED";
	case CONNECTION_ERROR_OPERATION_FAILED:
		return "OPERATION_FAILED";
	case CONNECTION_ERROR_ITERATOR_END:
		return "ITERATOR_END";
	case CONNECTION_ERROR_NO_CONNECTION:
		return "NO_CONNECTION";
	case CONNECTION_ERROR_NOW_IN_PROGRESS:
		return "NOW_IN_PROGRESS";
	case CONNECTION_ERROR_ALREADY_EXISTS:
		return "ALREADY_EXISTS";
	case CONNECTION_ERROR_OPERATION_ABORTED:
		return "OPERATION_ABORTED";
	case CONNECTION_ERROR_DHCP_FAILED:
		return "DHCP_FAILED";
	case CONNECTION_ERROR_INVALID_KEY:
		return "INVALID_KEY";
	case CONNECTION_ERROR_NO_REPLY:
		return "NO_REPLY";
	case CONNECTION_ERROR_PERMISSION_DENIED:
		return "PERMISSION_DENIED";
	case CONNECTION_ERROR_NOT_SUPPORTED:
		return "NOT_SUPPORTED";
	case CONNECTION_ERROR_ALREADY_INITIALIZED:
		return "ALREADY_INITIALIZED";
	case CONNECTION_ERROR_NOT_INITIALIZED:
		return "NOT_INITIALIZED";
	}

	return "UNKNOWN";
}

static const char *__libnet_convert_cp_state_to_string(connection_profile_state_e state)
{
	switch (state) {
	case CONNECTION_PROFILE_STATE_DISCONNECTED:
		return "DISCONNECTED";
	case CONNECTION_PROFILE_STATE_ASSOCIATION:
		return "ASSOCIATION";
	case CONNECTION_PROFILE_STATE_CONFIGURATION:
		return "CONFIGURATION";
	case CONNECTION_PROFILE_STATE_CONNECTED:
		return "CONNECTED";
	default:
		return "UNKNOWN";
	}
}

static void __libnet_state_changed_cb(char *profile_name, connection_profile_state_e state)
{
	struct _profile_cb_s *cb_info;

	if (profile_name == NULL)
		return;

	cb_info = g_hash_table_lookup(profile_cb_table, profile_name);
	if (cb_info == NULL)
		return;

	if (cb_info->state == state)
		return;

	cb_info->state = state;

	if (state < 0 || cb_info->callback == NULL)
		return;

	cb_info->callback(cb_info->state, cb_info->user_data);
}

static void __libnet_clear_profile_list(struct _profile_list_s *profile_list)
{
	if (profile_list->count > 0)
		g_free(profile_list->profiles);

	profile_list->count = 0;
	profile_list->next = 0;
	profile_list->profiles = NULL;
}

static void __libnet_evt_cb(net_event_info_t *event_cb, void *user_data)
{
	bool is_requested = false;
	connection_error_e result = CONNECTION_ERROR_NONE;
	connection_handle_s *conn_handle = (connection_handle_s *)user_data;

	switch (event_cb->Event) {
	case NET_EVENT_OPEN_RSP:
		is_requested = true;
		/* fall through */
	case NET_EVENT_OPEN_IND:
		result = __libnet_convert_to_cp_error_type(event_cb->Error);
		CONNECTION_LOG(CONNECTION_INFO, "Connection opened %s[%s]",
					(is_requested) ? "RSP" : "IND",
					__libnet_convert_cp_error_type_to_string(result));

		if (is_requested) {
			if (conn_handle->opened_callback) {
				conn_handle->opened_callback(result,
					conn_handle->opened_user_data);

				conn_handle->opened_callback = NULL;
				conn_handle->opened_user_data = NULL;
			}
		}

		switch (event_cb->Error) {
		case NET_ERR_NONE:
		case NET_ERR_ACTIVE_CONNECTION_EXISTS:
			CONNECTION_LOG(CONNECTION_INFO, "Successfully open connection");

			__libnet_state_changed_cb(event_cb->ProfileName, CONNECTION_PROFILE_STATE_CONNECTED);
			return;
		default:
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to open connection[%s]",
						__libnet_convert_cp_error_type_to_string(result));
		}

		__libnet_state_changed_cb(event_cb->ProfileName, CONNECTION_PROFILE_STATE_DISCONNECTED);

		break;
	case NET_EVENT_CLOSE_RSP:
		is_requested = true;
		/* fall through */
	case NET_EVENT_CLOSE_IND:
		result = __libnet_convert_to_cp_error_type(event_cb->Error);
		CONNECTION_LOG(CONNECTION_INFO, "Connection closed %s[%s]",
					(is_requested) ? "RSP" : "IND",
					__libnet_convert_cp_error_type_to_string(result));

		if (is_requested) {
			if (conn_handle->closed_callback) {
				conn_handle->closed_callback(result,
					conn_handle->closed_user_data);

				conn_handle->closed_callback = NULL;
				conn_handle->closed_user_data = NULL;
			}
		}

		switch (event_cb->Error) {
		case NET_ERR_NONE:
			CONNECTION_LOG(CONNECTION_INFO, "Successfully closed connection");

			__libnet_state_changed_cb(event_cb->ProfileName, CONNECTION_PROFILE_STATE_DISCONNECTED);
			return;
		default:
			CONNECTION_LOG(CONNECTION_ERROR, "Failed to close connection[%s]",
							__libnet_convert_cp_error_type_to_string(result));
		}

		break;
	case NET_EVENT_NET_STATE_IND:
		CONNECTION_LOG(CONNECTION_INFO, "State changed IND");

		if (event_cb->Datalength != sizeof(net_state_type_t))
			return;

		net_state_type_t *profile_state = (net_state_type_t *)event_cb->Data;
		connection_profile_state_e cp_state = _profile_convert_to_cp_state(*profile_state);

		CONNECTION_LOG(CONNECTION_INFO, "state: %s", __libnet_convert_cp_state_to_string(cp_state));
		SECURE_CONNECTION_LOG(CONNECTION_INFO, "profile name: %s", event_cb->ProfileName);

		__libnet_state_changed_cb(event_cb->ProfileName, cp_state);

		break;
	case NET_EVENT_CELLULAR_SET_DEFAULT_RSP:
		result = __libnet_convert_to_cp_error_type(event_cb->Error);
		CONNECTION_LOG(CONNECTION_INFO, "Got set default profile RSP %d", result);
		if (conn_handle->set_default_callback) {
			conn_handle->set_default_callback(result,
				conn_handle->set_default_user_data);

			conn_handle->set_default_callback = NULL;
			conn_handle->set_default_user_data = NULL;
		}
		break;
	case NET_EVENT_CELLULAR_RESET_DEFAULT_RSP:
		result = __libnet_convert_to_cp_error_type(event_cb->Error);
		CONNECTION_LOG(CONNECTION_INFO, "Got reset default profile RSP %d", result);
		if (conn_handle->reset_callback) {
			conn_handle->reset_callback(result,
				conn_handle->reset_user_data);

			conn_handle->reset_callback = NULL;
			conn_handle->reset_user_data = NULL;
		}
		break;
	case NET_EVENT_ETHERNET_CABLE_ATTACHED:
		CONNECTION_LOG(CONNECTION_INFO, "Got Ethernet cable Attached Indication\n");
		if (conn_handle->ethernet_cable_state_changed_callback) {
			conn_handle->ethernet_cable_state_changed_callback(CONNECTION_ETHERNET_CABLE_ATTACHED,
				conn_handle->ethernet_cable_state_changed_user_data);
		}
		break;
	case NET_EVENT_ETHERNET_CABLE_DETACHED:
		CONNECTION_LOG(CONNECTION_INFO, "Got Ethernet cable detached Indication\n");
		if (conn_handle->ethernet_cable_state_changed_callback) {
			conn_handle->ethernet_cable_state_changed_callback(CONNECTION_ETHERNET_CABLE_DETACHED,
				conn_handle->ethernet_cable_state_changed_user_data);
		}
		break;
	case NET_EVENT_NETWORK_TYPE_CHANGED:
		CONNECTION_LOG(CONNECTION_INFO, "Got Network Type Changed Indication");
		int *state = (int *) event_cb->Data;
		if (conn_handle->type_changed_callback) {
			int type = CONNECTION_TYPE_DISCONNECTED;

			switch (*state) {
			case VCONFKEY_NETWORK_CELLULAR:
				type = CONNECTION_TYPE_CELLULAR;
				break;
			case VCONFKEY_NETWORK_WIFI:
				type = CONNECTION_TYPE_WIFI;
				break;
			case VCONFKEY_NETWORK_ETHERNET:
				type = CONNECTION_TYPE_ETHERNET;
				break;
			case VCONFKEY_NETWORK_BLUETOOTH:
				type = CONNECTION_TYPE_BT;
				break;
			case VCONFKEY_NETWORK_DEFAULT_PROXY:
				type = CONNECTION_TYPE_NET_PROXY;
				break;
			default:
				type = CONNECTION_TYPE_DISCONNECTED;
				break;
			}

			conn_handle->type_changed_callback(type,
				conn_handle->type_changed_user_data);
		}
		break;
	case NET_EVENT_IPV4_ADDRESS_CHANGED:
		CONNECTION_LOG(CONNECTION_INFO, "Got IPv4 Address Changed Indication");
		if (conn_handle->ip_changed_callback) {
			char *ipv4_addr = NULL;
			char *ipv6_addr = NULL;
			char *addr = (char *)event_cb->Data;

			ipv4_addr = g_strdup(addr);
			ipv6_addr = vconf_get_str(VCONFKEY_NETWORK_IP6);
			if (ipv6_addr == NULL)
				CONNECTION_LOG(CONNECTION_ERROR, //LCOV_EXCL_LINE
						"vconf_get_str(VCONFKEY_NETWORK_IP6) failed");

			conn_handle->ip_changed_callback(ipv4_addr, ipv6_addr,
				conn_handle->ip_changed_user_data);

			g_free(ipv4_addr);
			g_free(ipv6_addr);
		}
		break;
	case NET_EVENT_IPV6_ADDRESS_CHANGED:
		CONNECTION_LOG(CONNECTION_INFO, "Got IPv6 Address Changed Indication");
		if (conn_handle->ip_changed_callback) {
			char *ipv4_addr = NULL;
			char *ipv6_addr = NULL;
			char *addr = (char *)event_cb->Data;

			ipv6_addr = g_strdup(addr);
			ipv4_addr = vconf_get_str(VCONFKEY_NETWORK_IP);
			if (ipv4_addr == NULL)
				CONNECTION_LOG(CONNECTION_ERROR, //LCOV_EXCL_LINE
						"vconf_get_str(VCONFKEY_NETWORK_IP) failed");

			conn_handle->ip_changed_callback(ipv4_addr, ipv6_addr,
				conn_handle->ip_changed_user_data);

			g_free(ipv4_addr);
			g_free(ipv6_addr);
		}
		break;
	case NET_EVENT_PROXY_ADDRESS_CHANGED:
		CONNECTION_LOG(CONNECTION_INFO, "Got Proxy Changed Indication");
		char *proxy_addr = (char *)event_cb->Data;

		if (conn_handle->proxy_changed_callback) {
			conn_handle->proxy_changed_callback(proxy_addr, NULL,
				conn_handle->proxy_changed_user_data);
		}
		break;
	case NET_EVENT_INTERNET_ONLINE_IND:
	case NET_EVENT_INTERNET_OFFLINE_IND:
		CONNECTION_LOG(CONNECTION_INFO, "Got Internet State Changed Indication: %s",
				event_cb->Event == NET_EVENT_INTERNET_ONLINE_IND ? "Online" : "Offline");
		net_device_t *device_type = (net_device_t *) event_cb->Data;

		if (conn_handle->internet_state_changed_callback) {
			net_profile_info_t active_profile;
			int rv;

			rv = net_get_active_net_info(conn_handle->network_info_handle, &active_profile);

			if (rv == NET_ERR_NO_SERVICE && event_cb->Event == NET_EVENT_INTERNET_OFFLINE_IND) {
				conn_handle->internet_state_changed_callback(CONNECTION_INTERNET_STATE_OFFLINE,
					conn_handle->internet_state_changed_user_data); //LCOV_EXCL_LINE
				break;
			} else if (rv == NET_ERR_ACCESS_DENIED) {
				CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
				break;
			} else if (rv != NET_ERR_NONE) {
				CONNECTION_LOG(CONNECTION_ERROR, "Unable to get Default profile handle"); //LCOV_EXCL_LINE
				break; //LCOV_EXCL_LINE
			}

			if (event_cb->Event == NET_EVENT_INTERNET_ONLINE_IND) {
				if (active_profile.ProfileState == NET_STATE_TYPE_ONLINE &&
						active_profile.profile_type == *device_type)
					conn_handle->internet_state_changed_callback(CONNECTION_INTERNET_STATE_ONLINE,
							conn_handle->internet_state_changed_user_data);
			} else {
				if (active_profile.ProfileState != NET_STATE_TYPE_ONLINE)
					conn_handle->internet_state_changed_callback(CONNECTION_INTERNET_STATE_OFFLINE,
							conn_handle->internet_state_changed_user_data);
			}
		}
		break;

	default:
		break;
	}
}
//LCOV_EXCL_STOP

static int __libnet_get_connected_count(struct _profile_list_s *profile_list)
{
	int count = 0;
	int i = 0;

	for (; i < profile_list->count; i++) {
		if (profile_list->profiles[i].ProfileState == NET_STATE_TYPE_ONLINE ||
		    profile_list->profiles[i].ProfileState == NET_STATE_TYPE_READY)
			count++;
	}

	return count;
}

static void __libnet_copy_connected_profile(net_profile_info_t **dest, struct _profile_list_s *source)
{
	int i = 0;

	for (; i < source->count; i++) {
		if (source->profiles[i].ProfileState == NET_STATE_TYPE_ONLINE ||
		    source->profiles[i].ProfileState == NET_STATE_TYPE_READY) {
			memcpy(*dest, &source->profiles[i], sizeof(net_profile_info_t));
			(*dest)++;
		}
	}
}

//LCOV_EXCL_START
static int __libnet_get_default_count(struct _profile_list_s *profile_list)
{
	int count = 0;
	int i = 0;

	for (; i < profile_list->count; i++) {
		if (profile_list->profiles[i].profile_type == NET_DEVICE_CELLULAR) {
			if (profile_list->profiles[i].ProfileInfo.Pdp.DefaultConn == TRUE)
				count++;
		}
	}

	return count;
}

static void __libnet_copy_default_profile(net_profile_info_t **dest, struct _profile_list_s *source)
{
	int i = 0;

	for (; i < source->count; i++) {
		if (source->profiles[i].profile_type == NET_DEVICE_CELLULAR) {
			if (source->profiles[i].ProfileInfo.Pdp.DefaultConn == TRUE) {
				memcpy(*dest, &source->profiles[i], sizeof(net_profile_info_t));
				(*dest)++;
			}
		}
	}
}
//LCOV_EXCL_STOP

int _connection_libnet_init(connection_handle_s *conn_handle)
{
	int rv;

	rv = net_register_client(&(conn_handle->network_info_handle),
				(net_event_cb_t)__libnet_evt_cb, conn_handle);
	if (rv != NET_ERR_NONE)
		return rv;

	if (profile_cb_table == NULL)
		profile_cb_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

	return NET_ERR_NONE;
}

bool _connection_libnet_deinit(connection_handle_s *conn_handle, bool is_empty)
{
	net_deregister_client(conn_handle->network_info_handle);

	if (is_empty) {
		if (profile_cb_table) {
			g_hash_table_destroy(profile_cb_table);
			profile_cb_table = NULL;
		}

		__libnet_clear_profile_list(&profile_iterator);

		if (prof_handle_list) {
			g_slist_free_full(prof_handle_list, g_free);
			prof_handle_list = NULL;
		}
	}

	return true;
}

void _connection_set_cs_tid(int tid, connection_handle_s *conn_handle)
{
	net_set_cs_tid(tid, conn_handle->network_info_handle);
}

void _connection_unset_cs_tid(int tid, connection_handle_s *conn_handle)
{
	net_unset_cs_tid(tid, conn_handle->network_info_handle);
}

bool _connection_libnet_check_profile_validity(connection_profile_h profile)
{
	GSList *list;
	int i = 0;

	if (profile == NULL)
		return false;

	for (list = prof_handle_list; list; list = list->next)
		if (profile == list->data) return true;

	for (; i < profile_iterator.count; i++)
		if (profile == &profile_iterator.profiles[i]) return true;

	return false;
}

int _connection_libnet_get_metered_state(connection_handle_s *conn_handle, bool* is_metered)
{
	int rv = 0;
	int status = 0;

	rv = net_get_metered_state(conn_handle->network_info_handle, &status);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");
		return CONNECTION_ERROR_PERMISSION_DENIED;
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get metered state[%d]", rv);
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	if (status == 1)
		*is_metered = true;
	else
		*is_metered = false;
	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_wifi_state(connection_handle_s *conn_handle, connection_wifi_state_e *state)
{
	int rv;
	net_wifi_state_t wlan_state;

	rv = net_get_wifi_state(conn_handle->network_info_handle, &wlan_state);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get Wi-Fi state[%d]", rv); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	switch (wlan_state) {
	case WIFI_OFF:
		*state = CONNECTION_WIFI_STATE_DEACTIVATED;
		break;
	case WIFI_ON:
	case WIFI_ASSOCIATION:
	case WIFI_CONFIGURATION:
		*state = CONNECTION_WIFI_STATE_DISCONNECTED;
		break;
	case WIFI_CONNECTED:
	case WIFI_DISCONNECTING:
		*state = CONNECTION_WIFI_STATE_CONNECTED;
		break;
	default:
		CONNECTION_LOG(CONNECTION_ERROR, "Unknown Wi-Fi state"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_OPERATION; //LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

//LCOV_EXCL_START
int _connection_libnet_get_ethernet_state(connection_handle_s *conn_handle,
			connection_ethernet_state_e *state)
{
	int rv;
	struct _profile_list_s ethernet_profiles = {0, 0, NULL};
	rv = net_get_profile_list(conn_handle->network_info_handle,
			NET_DEVICE_ETHERNET, &ethernet_profiles.profiles,
			&ethernet_profiles.count);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");
		return CONNECTION_ERROR_PERMISSION_DENIED;
	}

	if (ethernet_profiles.count == 0) {
		*state = CONNECTION_ETHERNET_STATE_DEACTIVATED;
		return CONNECTION_ERROR_NONE;
	}

	switch (ethernet_profiles.profiles->ProfileState) {
	case NET_STATE_TYPE_ONLINE:
	case NET_STATE_TYPE_READY:
		*state = CONNECTION_ETHERNET_STATE_CONNECTED;
		break;
	case NET_STATE_TYPE_IDLE:
	case NET_STATE_TYPE_FAILURE:
	case NET_STATE_TYPE_ASSOCIATION:
	case NET_STATE_TYPE_CONFIGURATION:
	case NET_STATE_TYPE_DISCONNECT:
		*state = CONNECTION_ETHERNET_STATE_DISCONNECTED;
		break;
	default:
		__libnet_clear_profile_list(&ethernet_profiles);
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	__libnet_clear_profile_list(&ethernet_profiles);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_ethernet_cable_state(connection_handle_s *conn_handle,
			connection_ethernet_cable_state_e* state)
{
	int rv = 0;
	int status = 0;

	rv = net_get_ethernet_cable_state(conn_handle->network_info_handle, &status);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");
		return CONNECTION_ERROR_PERMISSION_DENIED;
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get ethernet cable state[%d]", rv);
		return CONNECTION_ERROR_OPERATION_FAILED;
	}

	if (status == 1)
		*state = CONNECTION_ETHERNET_CABLE_ATTACHED;
	else
		*state = CONNECTION_ETHERNET_CABLE_DETACHED;
	return CONNECTION_ERROR_NONE;
}
//LCOV_EXCL_STOP

int _connection_libnet_get_bluetooth_state(connection_handle_s *conn_handle, connection_bt_state_e *state)
{
	int i = 0;
	int rv = 0;
	struct _profile_list_s bluetooth_profiles = {0, 0, NULL};
	rv = net_get_profile_list(conn_handle->network_info_handle,
			NET_DEVICE_BLUETOOTH, &bluetooth_profiles.profiles,
			&bluetooth_profiles.count);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	}

	if (bluetooth_profiles.count == 0) {
		*state = CONNECTION_BT_STATE_DEACTIVATED;
		return CONNECTION_ERROR_NONE;
	}

	//LCOV_EXCL_START
	for (; i < bluetooth_profiles.count; i++) {
		switch (bluetooth_profiles.profiles[i].ProfileState) {
		case NET_STATE_TYPE_ONLINE:
		case NET_STATE_TYPE_READY:
			*state = CONNECTION_BT_STATE_CONNECTED;
			goto done;
		case NET_STATE_TYPE_IDLE:
		case NET_STATE_TYPE_FAILURE:
		case NET_STATE_TYPE_ASSOCIATION:
		case NET_STATE_TYPE_CONFIGURATION:
		case NET_STATE_TYPE_DISCONNECT:
			*state = CONNECTION_BT_STATE_DISCONNECTED;
			break;
		default:
			__libnet_clear_profile_list(&bluetooth_profiles);
			return CONNECTION_ERROR_OPERATION_FAILED;
		}
	}
	//LCOV_EXCL_STOP

done:
	__libnet_clear_profile_list(&bluetooth_profiles);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_profile_iterator(connection_handle_s *conn_handle,
			connection_iterator_type_e type, connection_profile_iterator_h* profile_iter_h)
{
	int count = 0;
	int rv;
	net_profile_info_t *profiles = NULL;

	struct _profile_list_s profile_list = {0, 0, NULL};

	__libnet_clear_profile_list(&profile_iterator);

	rv = net_get_all_profile_list(conn_handle->network_info_handle,
			&profile_list.profiles, &profile_list.count);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NO_SERVICE && rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	*profile_iter_h = &profile_iterator;

	switch (type) {
	case CONNECTION_ITERATOR_TYPE_REGISTERED:
		count = profile_list.count;
		CONNECTION_LOG(CONNECTION_INFO, "Total profile count : %d", count);
		if (count == 0)
			return CONNECTION_ERROR_NONE;

		profiles = g_try_new0(net_profile_info_t, count);
		if (profiles == NULL) {
			__libnet_clear_profile_list(&profile_list);
			return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
		}

		profile_iterator.profiles = profiles;

		memcpy(profiles, profile_list.profiles, sizeof(net_profile_info_t) * count);

		break;
	case CONNECTION_ITERATOR_TYPE_CONNECTED:
		count = __libnet_get_connected_count(&profile_list);
		CONNECTION_LOG(CONNECTION_INFO, "Total connected profile count : %d", count);
		if (count == 0)
			return CONNECTION_ERROR_NONE;

		profiles = g_try_new0(net_profile_info_t, count);
		if (profiles == NULL) {
			__libnet_clear_profile_list(&profile_list);
			return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
		}

		profile_iterator.profiles = profiles;

		__libnet_copy_connected_profile(&profiles, &profile_list);

		break;
	case CONNECTION_ITERATOR_TYPE_DEFAULT:
		count = __libnet_get_default_count(&profile_list);
		CONNECTION_LOG(CONNECTION_INFO, "Total default profile count : %d", count); //LCOV_EXCL_LINE
		if (count == 0)
			return CONNECTION_ERROR_NONE;

		profiles = g_try_new0(net_profile_info_t, count);
		if (profiles == NULL) {
			__libnet_clear_profile_list(&profile_list);
			return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
		}

		profile_iterator.profiles = profiles;

		__libnet_copy_default_profile(&profiles, &profile_list);

		break;
	}

	__libnet_clear_profile_list(&profile_list);

	profile_iterator.count = count;

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_iterator_next(connection_profile_iterator_h profile_iter_h, connection_profile_h *profile)
{
	if (profile_iter_h != &profile_iterator)
		return CONNECTION_ERROR_INVALID_PARAMETER;

	if (profile_iterator.count <= profile_iterator.next)
		return CONNECTION_ERROR_ITERATOR_END;

	*profile = &profile_iterator.profiles[profile_iterator.next];
	profile_iterator.next++;

	return CONNECTION_ERROR_NONE;
}

bool _connection_libnet_iterator_has_next(connection_profile_iterator_h profile_iter_h)
{
	if (profile_iter_h != &profile_iterator)
		return false;

	if (profile_iterator.count <= profile_iterator.next)
		return false;

	return true;
}

int _connection_libnet_destroy_iterator(connection_profile_iterator_h profile_iter_h)
{
	if (profile_iter_h != &profile_iterator)
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE

	__libnet_clear_profile_list(&profile_iterator);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_current_profile(connection_handle_s *conn_handle,
			connection_profile_h *profile)
{
	net_profile_info_t active_profile;
	int rv;

	rv = net_get_active_net_info(conn_handle->network_info_handle, &active_profile);
	if (rv == NET_ERR_NO_SERVICE)
		return CONNECTION_ERROR_NO_CONNECTION; //LCOV_EXCL_LINE
	else if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	*profile = g_try_malloc0(sizeof(net_profile_info_t));
	if (*profile == NULL)
		return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE

	memcpy(*profile, &active_profile, sizeof(net_profile_info_t));
	prof_handle_list = g_slist_append(prof_handle_list, *profile);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_reset_profile(connection_handle_s *conn_handle,
		connection_reset_option_e type, connection_cellular_subscriber_id_e id)
{
	int rv;

	rv = net_reset_profile(conn_handle->network_info_handle, type, id);
	if (rv == NET_ERR_ACCESS_DENIED)
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_open_profile(connection_handle_s *conn_handle,
			connection_profile_h profile)
{
	int rv;

	if (!(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	net_profile_info_t *profile_info = profile;

	if (profile_info->profile_type == NET_DEVICE_MESH)
		rv = net_open_mesh_connection_with_profile(conn_handle->network_info_handle,
				profile_info->ProfileName);
	else
		rv = net_open_connection_with_profile(conn_handle->network_info_handle,
				profile_info->ProfileName);

	if (rv == NET_ERR_ACCESS_DENIED)
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_cellular_service_profile(connection_handle_s *conn_handle,
		connection_cellular_service_type_e type, connection_profile_h *profile)
{
	int i = 0, j = 0;
	int rv = NET_ERR_NONE;
#if defined TIZEN_DUALSIM_ENABLE
	int default_subscriber_id = 0;
	char subscriber_id[3];
#endif

	struct _profile_list_s cellular_profiles = { 0, 0, NULL };
	net_service_type_t service_type = _connection_profile_convert_to_libnet_cellular_service_type(type);

	rv = net_get_profile_list(conn_handle->network_info_handle,
			NET_DEVICE_CELLULAR, &cellular_profiles.profiles,
			&cellular_profiles.count);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get profile list (%d)", rv); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

#if defined TIZEN_DUALSIM_ENABLE
	if (vconf_get_int(VCONF_TELEPHONY_DEFAULT_DATA_SERVICE,
						&default_subscriber_id) != 0) {
		CONNECTION_LOG(CONNECTION_ERROR,
						"Failed to get VCONF_TELEPHONY_DEFAULT_DATA_SERVICE");
		__libnet_clear_profile_list(&cellular_profiles); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	g_snprintf(subscriber_id, sizeof(subscriber_id), "%d", default_subscriber_id);
#endif

	for (i = 0; i < cellular_profiles.count; i++)
		if (cellular_profiles.profiles[i].ProfileInfo.Pdp.ServiceType == service_type)
#if defined TIZEN_DUALSIM_ENABLE
			if (g_str_has_suffix(
					cellular_profiles.profiles[i].ProfileInfo.Pdp.PSModemPath,
					subscriber_id) == TRUE)
#endif
				break;

	if (i >= cellular_profiles.count) {
		__libnet_clear_profile_list(&cellular_profiles); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	*profile = g_try_malloc0(sizeof(net_profile_info_t));
	if (*profile == NULL) {
		__libnet_clear_profile_list(&cellular_profiles); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OUT_OF_MEMORY; //LCOV_EXCL_LINE
	}

	memcpy(*profile, &cellular_profiles.profiles[i], sizeof(net_profile_info_t));

	if (cellular_profiles.profiles[i].ProfileInfo.Pdp.DefaultConn)
		goto done;

	//LCOV_EXCL_START
	if (type != CONNECTION_CELLULAR_SERVICE_TYPE_INTERNET &&
	    type != CONNECTION_CELLULAR_SERVICE_TYPE_PREPAID_INTERNET)
		goto done;

	for (j = 0; j < cellular_profiles.count; j++) {
		if (i == j)
			continue;

		if (cellular_profiles.profiles[j].ProfileInfo.Pdp.ServiceType != service_type)
			continue;

		if (cellular_profiles.profiles[j].ProfileInfo.Pdp.DefaultConn) {
			memcpy(*profile, &cellular_profiles.profiles[j], sizeof(net_profile_info_t));
			goto done;
		}
	}
	//LCOV_EXCL_STOP

done:
	__libnet_clear_profile_list(&cellular_profiles);
	prof_handle_list = g_slist_append(prof_handle_list, *profile);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_set_cellular_service_profile_sync(connection_handle_s *conn_handle,
			connection_cellular_service_type_e type, connection_profile_h profile)
{
	int rv;

	if (!(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	net_profile_info_t *profile_info = profile;
	connection_cellular_service_type_e service_type;

	service_type = _profile_convert_to_connection_cellular_service_type(profile_info->ProfileInfo.Pdp.ServiceType);

	if (service_type != type)
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE

	rv = net_set_default_cellular_service_profile(conn_handle->network_info_handle,
				profile_info->ProfileName);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_set_cellular_service_profile_async(connection_handle_s *conn_handle,
			connection_cellular_service_type_e type, connection_profile_h profile)
{
	int rv;

	if (!(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	net_profile_info_t *profile_info = profile;
	connection_cellular_service_type_e service_type;

	service_type = _profile_convert_to_connection_cellular_service_type(profile_info->ProfileInfo.Pdp.ServiceType);

	if (service_type != type)
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE

	rv = net_set_default_cellular_service_profile_async(conn_handle->network_info_handle,
				profile_info->ProfileName);
	if (rv == NET_ERR_ACCESS_DENIED) {
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_close_profile(connection_handle_s *conn_handle, connection_profile_h profile)
{
	int rv;

	if (!(_connection_libnet_check_profile_validity(profile))) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid parameter"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	net_profile_info_t *profile_info = profile;

	if (profile_info->profile_type == NET_DEVICE_MESH)
		rv = net_close_mesh_connection(conn_handle->network_info_handle, profile_info->ProfileName);
	else
		rv = net_close_connection(conn_handle->network_info_handle, profile_info->ProfileName);

	if (rv == NET_ERR_ACCESS_DENIED)
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_add_route(connection_handle_s *conn_handle,
			const char *interface_name, const char *host_address)
{
	int rv;
	char *endstr = NULL;
	int address_family = 0;

	address_family = AF_INET;

	endstr = strrchr(host_address, '.');
	if (endstr == NULL ||
			strcmp(endstr, ".0") == 0 ||
			strncmp(host_address, "0.", 2) == 0 ||
			strstr(host_address, "255") != NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_add_route(conn_handle->network_info_handle,
				host_address, interface_name, address_family);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_remove_route(connection_handle_s *conn_handle,
			const char *interface_name, const char *host_address)
{
	int rv;
	char *endstr = strrchr(host_address, '.');
	int address_family = 0;

	address_family = AF_INET;

	endstr = strrchr(host_address, '.');
	if (endstr == NULL ||
		strcmp(endstr, ".0") == 0 ||
		strncmp(host_address, "0.", 2) == 0 ||
		strstr(host_address, ".0.") != NULL || strstr(host_address, "255") != NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_remove_route(conn_handle->network_info_handle,
				host_address, interface_name, address_family);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_add_route_ipv6(connection_handle_s *conn_handle,
			const char *interface_name, const char *host_address, const char *gateway)
{
	int rv;
	int address_family = 0;

	address_family = AF_INET6;

	if (strncmp(host_address, "fe80:", 5) == 0 ||
		strncmp(host_address, "ff00:", 5) == 0 ||
		strncmp(host_address, "::", 2) == 0) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_add_route_ipv6(conn_handle->network_info_handle,
				host_address, interface_name, address_family, gateway);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_remove_route_ipv6(connection_handle_s *conn_handle,
			const char *interface_name, const char *host_address, const char *gateway)
{
	int rv;
	int address_family = 0;

	address_family = AF_INET6;

	if (strncmp(host_address, "fe80:", 5) == 0 ||
		strncmp(host_address, "ff00:", 5) == 0 ||
		strncmp(host_address, "::", 2) == 0) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
	}

	rv = net_remove_route_ipv6(conn_handle->network_info_handle,
				host_address, interface_name, address_family, gateway);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_add_route_entry(connection_handle_s *conn_handle,
		connection_address_family_e address_family, const char *interface_name,
		const char *host_address, const char *gateway)
{
	int rv;
	char *endstr = NULL;
	int address_family_type = 0;

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4)
		address_family_type = AF_INET;
	else
		address_family_type = AF_INET6;

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4) {

		endstr = strrchr(host_address, '.');
		if (endstr == NULL ||
				strcmp(endstr, ".0") == 0 ||
				strncmp(host_address, "0.", 2) == 0 ||
				strstr(host_address, "255") != NULL) {
			CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
		}

		rv = net_add_route_entry(conn_handle->network_info_handle,
				host_address, interface_name, address_family_type, gateway);
		if (rv == NET_ERR_ACCESS_DENIED) {
			CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
		} else if (rv != NET_ERR_NONE)
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	} else {

		if (strncmp(host_address, "fe80:", 5) == 0 ||
			strncmp(host_address, "ff00:", 5) == 0 ||
			strncmp(host_address, "::", 2) == 0) {
			CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
		}

		rv = net_add_route_ipv6(conn_handle->network_info_handle,
				host_address, interface_name, address_family_type, gateway);
		if (rv == NET_ERR_ACCESS_DENIED) {
			CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
		} else if (rv != NET_ERR_NONE)
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_remove_route_entry(connection_handle_s *conn_handle,
		connection_address_family_e address_family, const char *interface_name,
		const char *host_address, const char *gateway)
{
	int rv;
	char *endstr = strrchr(host_address, '.');
	int address_family_type = 0;

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4)
		address_family_type = AF_INET;
	else
		address_family_type = AF_INET6;

	if (address_family == CONNECTION_ADDRESS_FAMILY_IPV4) {
		endstr = strrchr(host_address, '.');
		if (endstr == NULL ||
			strcmp(endstr, ".0") == 0 ||
			strncmp(host_address, "0.", 2) == 0 ||
			strstr(host_address, ".0.") != NULL || strstr(host_address, "255") != NULL) {
			CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
		}

		rv = net_remove_route_entry(conn_handle->network_info_handle, host_address,
					interface_name, address_family_type, gateway);
		if (rv == NET_ERR_ACCESS_DENIED) {
			CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
		} else if (rv != NET_ERR_NONE)
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	} else {

		if (strncmp(host_address, "fe80:", 5) == 0 ||
			strncmp(host_address, "ff00:", 5) == 0 ||
			strncmp(host_address, "::", 2) == 0) {
			CONNECTION_LOG(CONNECTION_ERROR, "Invalid IP address Passed\n"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_INVALID_PARAMETER; //LCOV_EXCL_LINE
		}

		rv = net_remove_route_ipv6(conn_handle->network_info_handle, host_address,
					interface_name, address_family_type, gateway);
		if (rv == NET_ERR_ACCESS_DENIED) {
			CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
		} else if (rv != NET_ERR_NONE)
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

void _connection_libnet_add_to_profile_list(connection_profile_h profile)
{
	prof_handle_list = g_slist_append(prof_handle_list, profile);
}

void _connection_libnet_remove_from_profile_list(connection_profile_h profile)
{
	prof_handle_list = g_slist_remove(prof_handle_list, profile);
	g_free(profile);
}

bool _connection_libnet_add_to_profile_cb_list(connection_profile_h profile,
		connection_profile_state_changed_cb callback, void *user_data)
{
	net_profile_info_t *profile_info = profile;
	char *profile_name = g_strdup(profile_info->ProfileName);

	struct _profile_cb_s *profile_cb_info = g_try_malloc0(sizeof(struct _profile_cb_s));
	if (profile_cb_info == NULL) {
		g_free(profile_name); //LCOV_EXCL_LINE
		return false; //LCOV_EXCL_LINE
	}

	profile_cb_info->callback = callback;
	profile_cb_info->user_data = user_data;
	profile_cb_info->state = _profile_convert_to_cp_state(profile_info->ProfileState);

	g_hash_table_replace(profile_cb_table, profile_name, profile_cb_info);

	return true;
}

bool _connection_libnet_remove_from_profile_cb_list(connection_profile_h profile)
{
	net_profile_info_t *profile_info = profile;

	if (g_hash_table_remove(profile_cb_table, profile_info->ProfileName) == TRUE)
		return true;

	return false; //LCOV_EXCL_LINE
}

int _connection_libnet_set_statistics(connection_handle_s *conn_handle,
			net_device_t device_type, net_statistics_type_e statistics_type)
{
	int rv;
	rv = net_set_statistics(conn_handle->network_info_handle,
				device_type, statistics_type);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_get_statistics(connection_handle_s *conn_handle,
			net_statistics_type_e statistics_type, unsigned long long *size)
{
	int rv;
	rv = net_get_statistics(conn_handle->network_info_handle,
				NET_DEVICE_WIFI, statistics_type, size);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_set_cellular_subscriber_id(connection_profile_h profile,
		connection_cellular_subscriber_id_e sim_id)
{
	char *modem_path = NULL;
	net_profile_info_t *profile_info = (net_profile_info_t *)profile;

	if (net_get_cellular_modem_object_path(&modem_path, sim_id) != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get subscriber[%d]", sim_id); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	if (!modem_path) {
		CONNECTION_LOG(CONNECTION_ERROR, "NULL modem object path"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
	}

	g_strlcpy(profile_info->ProfileInfo.Pdp.PSModemPath, modem_path,
				NET_PROFILE_NAME_LEN_MAX);
	g_free(modem_path);

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_check_get_privilege(void)
{
	int rv;

	rv = net_check_get_privilege();
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_check_profile_privilege(void)
{
	int rv;

	rv = net_check_profile_privilege();
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied"); //LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED; //LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE)
		return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE

	return CONNECTION_ERROR_NONE;
}

bool __libnet_check_feature_supported(const char *key, connection_supported_feature_e feature)
{
	if (!connection_is_feature_checked[feature]) {
		if (system_info_get_platform_bool(key, &connection_feature_supported[feature]) < 0) {
			CONNECTION_LOG(CONNECTION_ERROR, "Error - Feature getting from System Info"); //LCOV_EXCL_LINE
			set_last_result(CONNECTION_ERROR_OPERATION_FAILED); //LCOV_EXCL_LINE
			return CONNECTION_ERROR_OPERATION_FAILED; //LCOV_EXCL_LINE
		}
		connection_is_feature_checked[feature] = true;
	}
	return connection_feature_supported[feature];
}

int _connection_check_feature_supported(const char *feature_name, ...)
{
	va_list list;
	const char *key;
	bool value = false;
	bool feature_supported = false;

	va_start(list, feature_name);
	key = feature_name;
	while (1) {
		if (strcmp(key, TELEPHONY_FEATURE) == 0)
			value = __libnet_check_feature_supported(key, CONNECTION_SUPPORTED_FEATURE_TELEPHONY);
		if (strcmp(key, WIFI_FEATURE) == 0)
			value = __libnet_check_feature_supported(key, CONNECTION_SUPPORTED_FEATURE_WIFI);
		if (strcmp(key, TETHERING_BLUETOOTH_FEATURE) == 0)
			value = __libnet_check_feature_supported(key, CONNECTION_SUPPORTED_FEATURE_TETHERING_BLUETOOTH);
		if (strcmp(key, ETHERNET_FEATURE) == 0)
			value = __libnet_check_feature_supported(key, CONNECTION_SUPPORTED_FEATURE_ETHERNET);

		feature_supported |= value;
		key = va_arg(list, const char *);
		if (!key) break;
	}
	if (!feature_supported) {
		CONNECTION_LOG(CONNECTION_ERROR, "Error - Feature is not supported");
		set_last_result(CONNECTION_ERROR_NOT_SUPPORTED);
		va_end(list);
		return CONNECTION_ERROR_NOT_SUPPORTED;
	}

	va_end(list);
	set_last_result(CONNECTION_ERROR_NONE);
	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_start_tcpdump(connection_handle_s *conn_handle)
{
	connection_error_e result = CONNECTION_ERROR_NONE;
	net_err_t ret = NET_ERR_NONE;

	ret = net_start_tcpdump(conn_handle->network_info_handle);
	result = __libnet_convert_to_cp_error_type(ret);

	return result;
}

int _connection_libnet_stop_tcpdump(connection_handle_s *conn_handle)
{
	connection_error_e result = CONNECTION_ERROR_NONE;
	net_err_t ret = NET_ERR_NONE;

	ret = net_stop_tcpdump(conn_handle->network_info_handle);
	result = __libnet_convert_to_cp_error_type(ret);

	return result;
}

int _connection_libnet_get_tcpdump_state(connection_handle_s *conn_handle,
			gboolean *tcpdump_state)
{
	connection_error_e result = CONNECTION_ERROR_NONE;
	net_err_t ret = NET_ERR_NONE;

	ret = net_get_tcpdump_state(conn_handle->network_info_handle, tcpdump_state);
	result = __libnet_convert_to_cp_error_type(ret);

	return result;
}

void _connection_lock(void)
{
	pthread_mutex_lock(&g_conn_thread_mutex);
}

void _connection_unlock(void)
{
	pthread_mutex_unlock(&g_conn_thread_mutex);
}
