/*
 * Copyright (c) 2011-2013 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>

#include "net_connection_private.h"
#include "connection_extension.h"
#include "net_connection_mptcp_private.h"
#include "network-mptcp-intf.h"

static const char* __convert_mptcp_path_manager_enum_to_str(connection_mptcp_path_manager_e pm)
{
	switch (pm) {
	case CONNECTION_MPTCP_PM_UNKNOWN:
		return NULL;
	case CONNECTION_MPTCP_PM_DEFAULT:
		return "default";
	case CONNECTION_MPTCP_PM_FULLMESH:
		return "fullmesh";
	}
	return NULL;
}

static connection_mptcp_path_manager_e __convert_mptcp_path_manager_str_to_enum(char* pm)
{
	if (g_strcmp0(pm, "default") == 0)
		return CONNECTION_MPTCP_PM_DEFAULT;
	if (g_strcmp0(pm, "fullmesh") == 0)
		return CONNECTION_MPTCP_PM_FULLMESH;
	return CONNECTION_MPTCP_PM_UNKNOWN;	//LCOV_EXCL_LINE
}

static const char* __convert_mptcp_scheduler_enum_to_str(connection_mptcp_scheduler_e scheduler)
{
	switch (scheduler) {
	case CONNECTION_MPTCP_SCHEDULER_UNKNOWN:
		return NULL;
	case CONNECTION_MPTCP_SCHEDULER_DEFAULT:
		return "default";
	case CONNECTION_MPTCP_SCHEDULER_ROUNDROBIN:
		return "roundrobin";
	}
	return NULL;
}

static connection_mptcp_scheduler_e __convert_mptcp_scheduler_str_to_enum(char *scheduler)
{
	if (g_strcmp0(scheduler, "default") == 0)
		return CONNECTION_MPTCP_SCHEDULER_DEFAULT;
	if (g_strcmp0(scheduler, "roundrobin") == 0)
		return CONNECTION_MPTCP_SCHEDULER_ROUNDROBIN;
	return CONNECTION_MPTCP_SCHEDULER_UNKNOWN;	//LCOV_EXCL_LINE
}

gboolean _connection_libnet_mptcp_supported(connection_handle_s *conn_handle)
{
	int rv = 0;
	gboolean support = false;

	rv = net_mptcp_supported(conn_handle->network_info_handle, &support);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return false;
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to enable MPTCP [%d]", rv);	//LCOV_EXCL_LINE
		return false;
	}

	return support;
}

int _connection_libnet_mptcp_enable(connection_handle_s *conn_handle, connection_mptcp_enable_e enable)
{
	int rv = 0;

	rv = net_mptcp_set_enabled(conn_handle->network_info_handle, (int)enable);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to enable MPTCP [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_mptcp_get_enabled(connection_handle_s *conn_handle, connection_mptcp_enable_e* enable)
{
	int rv = 0;
	int result = 0;

	rv = net_mptcp_get_enabled(conn_handle->network_info_handle, &result);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get MPTCP enabled state [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	*enable = (connection_mptcp_enable_e)result;
	return CONNECTION_ERROR_NONE;
}


int _connection_libnet_mptcp_set_path_manager(connection_handle_s *conn_handle, connection_mptcp_path_manager_e pm)
{
	int rv = 0;
	const char* str = __convert_mptcp_path_manager_enum_to_str(pm);
	CONNECTION_LOG(CONNECTION_ERROR, "set path manager %s", str);

	if (str == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid Parameter");
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = net_mptcp_set_path_manager(conn_handle->network_info_handle, str);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to set MPTCP path manager [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_mptcp_get_path_manager(connection_handle_s *conn_handle, connection_mptcp_path_manager_e* pm)
{
	int rv = 0;
	char* result = NULL;

	rv = net_mptcp_get_path_manager(conn_handle->network_info_handle, &result);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get MPTCP path manager [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_ERROR, "get path manager %s", result);
	*pm = __convert_mptcp_path_manager_str_to_enum(result);
	g_free(result);
	return CONNECTION_ERROR_NONE;
}


int _connection_libnet_mptcp_set_scheduler(connection_handle_s *conn_handle, connection_mptcp_scheduler_e scheduler)
{
	int rv = 0;
	const char *str = __convert_mptcp_scheduler_enum_to_str(scheduler);
	CONNECTION_LOG(CONNECTION_ERROR, "set scheduler %s", str);

	if (str == NULL) {
		CONNECTION_LOG(CONNECTION_ERROR, "Invalid Parameter");
		return CONNECTION_ERROR_INVALID_PARAMETER;
	}

	rv = net_mptcp_set_scheduler(conn_handle->network_info_handle, str);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to set MPTCP scheduler [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	return CONNECTION_ERROR_NONE;
}

int _connection_libnet_mptcp_get_scheduler(connection_handle_s *conn_handle, connection_mptcp_scheduler_e* scheduler)
{
	int rv = 0;
	char* result = NULL;

	rv = net_mptcp_get_scheduler(conn_handle->network_info_handle, &result);
	if (rv == NET_ERR_ACCESS_DENIED) {
		CONNECTION_LOG(CONNECTION_ERROR, "Access denied");	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_PERMISSION_DENIED;	//LCOV_EXCL_LINE
	} else if (rv != NET_ERR_NONE) {
		CONNECTION_LOG(CONNECTION_ERROR, "Failed to get MPTCP scheduler [%d]", rv);	//LCOV_EXCL_LINE
		return CONNECTION_ERROR_OPERATION_FAILED;	//LCOV_EXCL_LINE
	}

	CONNECTION_LOG(CONNECTION_ERROR, "get scheduler %s", result);
	*scheduler = __convert_mptcp_scheduler_str_to_enum(result);
	g_free(result);
	return CONNECTION_ERROR_NONE;
}
